#!/usr/bin/env python
# pylint: disable=W0212

from agate import utils


def order_by(self, key, reverse=False):
    """
    Create a new table that is sorted.

    :param key:
        Either the name of a single column to sort by, a sequence of such
        names, or a :class:`function` that takes a row and returns a value
        to sort by.
    :param reverse:
        If `True` then sort in reverse (typically, descending) order.
    :returns:
        A new :class:`.Table`.
    """
    if len(self._rows) == 0:
        return self._fork(self._rows)

    key_is_row_function = hasattr(key, '__call__')
    key_is_sequence = utils.issequence(key)

    def sort_key(data):
        row = data[1]

        if key_is_row_function:
            k = key(row)
        elif key_is_sequence:
            k = tuple(utils.NullOrder() if row[n] is None else row[n] for n in key)
        else:
            k = row[key]

        if k is None:
            return utils.NullOrder()

        return k

    results = sorted(enumerate(self._rows), key=sort_key, reverse=reverse)

    indices, rows = zip(*results)

    if self._row_names is not None:
        row_names = [self._row_names[i] for i in indices]
    else:
        row_names = None

    return self._fork(rows, row_names=row_names)



#!/usr/bin/env python

import os
from collections import OrderedDict
from glob import glob

from agate.table import Table


@classmethod
def from_csv(cls, dir_path, column_names=None, column_types=None, row_names=None, header=True, **kwargs):
    """
    Create a new :class:`TableSet` from a directory of CSVs.

    See :meth:`.Table.from_csv` for additional details.

    :param dir_path:
        Path to a directory full of CSV files. All CSV files in this
        directory will be loaded.
    :param column_names:
        See :meth:`Table.__init__`.
    :param column_types:
        See :meth:`Table.__init__`.
    :param row_names:
        See :meth:`Table.__init__`.
    :param header:
        See :meth:`Table.from_csv`.
    """
    from agate.tableset import TableSet

    if not os.path.isdir(dir_path):
        raise IOError('Specified path doesn\'t exist or isn\'t a directory.')

    tables = OrderedDict()

    for path in glob(os.path.join(dir_path, '*.csv')):
        name = os.path.split(path)[1].strip('.csv')

        tables[name] = Table.from_csv(path, column_names, column_types, row_names=row_names, header=header, **kwargs)

    return TableSet(tables.values(), tables.keys())



#!/usr/bin/env python
# pylint: disable=W0212

import json
import os
from collections import OrderedDict


def to_json(self, path, key=None, newline=False, indent=None, **kwargs):
    """
    Write this table to a JSON file or file-like object.

    :code:`kwargs` will be passed through to the JSON encoder.

    :param path:
        File path or file-like object to write to.
    :param key:
        If specified, JSON will be output as an hash instead of a list. May
        be either the name of a column from the this table containing
        unique values or a :class:`function` that takes a row and returns
        a unique value.
    :param newline:
        If `True`, output will be in the form of "newline-delimited JSON".
    :param indent:
        If specified, the number of spaces to indent the JSON for
        formatting.
    """
    if key is not None and newline:
        raise ValueError('key and newline may not be specified together.')

    if newline and indent is not None:
        raise ValueError('newline and indent may not be specified together.')

    key_is_row_function = hasattr(key, '__call__')

    json_kwargs = {
        'ensure_ascii': False,
        'indent': indent
    }

    # Pass remaining kwargs through to JSON encoder
    json_kwargs.update(kwargs)

    json_funcs = [c.jsonify for c in self._column_types]

    close = True
    f = None

    try:
        if hasattr(path, 'write'):
            f = path
            close = False
        else:
            if os.path.dirname(path) and not os.path.exists(os.path.dirname(path)):
                os.makedirs(os.path.dirname(path))
            f = open(path, 'w')

        def dump_json(data):
            json.dump(data, f, **json_kwargs)

            if newline:
                f.write('\n')

        # Keyed
        if key is not None:
            output = OrderedDict()

            for row in self._rows:
                if key_is_row_function:
                    k = key(row)
                else:
                    k = str(row[key])

                if k in output:
                    raise ValueError('Value %s is not unique in the key column.' % str(k))

                values = tuple(json_funcs[i](d) for i, d in enumerate(row))
                output[k] = OrderedDict(zip(row.keys(), values))
            dump_json(output)
        # Newline-delimited
        elif newline:
            for row in self._rows:
                values = tuple(json_funcs[i](d) for i, d in enumerate(row))
                dump_json(OrderedDict(zip(row.keys(), values)))
        # Normal
        else:
            output = []

            for row in self._rows:
                values = tuple(json_funcs[i](d) for i, d in enumerate(row))
                output.append(OrderedDict(zip(row.keys(), values)))

            dump_json(output)
    finally:
        if close and f is not None:
            f.close()



#!/usr/bin/env python

from agate.aggregations import Aggregation
from agate.aggregations.has_nulls import HasNulls
from agate.aggregations.variance import PopulationVariance, Variance
from agate.data_types import Number
from agate.exceptions import DataTypeError
from agate.warns import warn_null_calculation


class StDev(Aggregation):
    """
    Calculate the sample standard of deviation of a column.

    For the population standard of deviation see :class:`.PopulationStDev`.

    :param column_name:
        The name of a column containing :class:`.Number` data.
    """
    def __init__(self, column_name):
        self._column_name = column_name
        self._variance = Variance(column_name)

    def get_aggregate_data_type(self, table):
        return Number()

    def validate(self, table):
        column = table.columns[self._column_name]

        if not isinstance(column.data_type, Number):
            raise DataTypeError('StDev can only be applied to columns containing Number data.')

        has_nulls = HasNulls(self._column_name).run(table)

        if has_nulls:
            warn_null_calculation(self, column)

    def run(self, table):
        variance = self._variance.run(table)
        if variance is not None:
            return variance.sqrt()


class PopulationStDev(StDev):
    """
    Calculate the population standard of deviation of a column.

    For the sample standard of deviation see :class:`.StDev`.

    :param column_name:
        The name of a column containing :class:`.Number` data.
    """
    def __init__(self, column_name):
        self._column_name = column_name
        self._population_variance = PopulationVariance(column_name)

    def get_aggregate_data_type(self, table):
        return Number()

    def validate(self, table):
        column = table.columns[self._column_name]

        if not isinstance(column.data_type, Number):
            raise DataTypeError('PopulationStDev can only be applied to columns containing Number data.')

        has_nulls = HasNulls(self._column_name).run(table)

        if has_nulls:
            warn_null_calculation(self, column)

    def run(self, table):
        variance = self._population_variance.run(table)
        if variance is not None:
            return variance.sqrt()
